/**
 *
 *  @class Restoreのメインとなるクラス。
 *  @classdesc Restore Main
 *
 */
var Restore = function() {
    this.file = null;
    this.isInvalid = false;
    this.errMessages = [];
    this.validater = new Validater();
    /**
    * ファイル形式のチェック（拡張子の簡易版）
    **/
    Restore.prototype.CheckFileFormet = function(event) {
        showBigLoader();
        let target = event.target;
        let file = target.files[0];
        let extension = file.name.split('.').slice(-1)[0].toLocaleLowerCase();
        if (extension !== 'json') {
            this.SetMessages("This file isn't JSON file.");
            this.ShowErrorMessage();
            return;
        }
        this.SetFile(file);
        this.CheckJsonValues();
    }
    /**
    * ファイルセッター
    * @param {blob} file
    **/
    Restore.prototype.SetFile = function(file) {
        this.file = file;
    }
    /**
    * ファイルゲッター
    * @return {blob} file
    **/
    Restore.prototype.GetFile = function() {
        return this.file;
    }
    /**
    * Invalidフラグセッター
    * @param {bool} bool
    **/
    Restore.prototype.SetInvalidFlag = function(bool) {
        this.isInvalid = bool;
    }
    /**
    * Invalidフラグゲッター
    **/
    Restore.prototype.GetInvalidFlag = function() {
        return this.isInvalid;
    }
    /**
    * エラーメッセージ格納
    * @param {string} msg
    **/
    Restore.prototype.SetMessages = function(msg) {
        this.errMessages.push(msg);
    }
    /**
    * エラーメッセージ取得
    * @return {array}
    **/
    Restore.prototype.GetMessages = function() {
        return this.errMessages;
    }
    /**
    * JSONの内容チェック
    **/
    Restore.prototype.CheckJsonValues = function() {
        let reader = new FileReader();
        let file = this.GetFile();
        let that = this;
        reader.onload = (function(){
            return function(event) {
                let jsonObj;
                try {
                    jsonObj = JSON.parse(event.target.result);
                } catch(e) {
                    that.SetMessages('This JSON file has format error.');
                    that.ShowErrorMessage();
                    return;
                }
                // 項目チェック（あるべき項目がundefinedならエラーと判定する）
                that.CheckObject(typeof jsonObj.Devices, 'Device');
                that.CheckObject(typeof jsonObj.Vlans, 'Vlan');
                if (that.GetInvalidFlag()) {
                    that.ShowErrorMessage();
                    return;
                }
                that.CheckObject(typeof jsonObj.Devices.Eth0, 'Device Eth0');
                that.CheckObject(typeof jsonObj.Devices.Eth1, 'Device Eth1');
                that.CheckObject(typeof jsonObj.Devices.Eth2, 'Device Eth0');
                that.CheckObject(typeof jsonObj.Vlans.Eth0, 'Vlan Eth0');
                that.CheckObject(typeof jsonObj.Vlans.Eth1, 'Vlan Eth1');
                that.CheckObject(typeof jsonObj.Vlans.Eth2, 'Vlan Eth2');
                if (that.GetInvalidFlag()) {
                    that.ShowErrorMessage();
                    return;
                }
                // バリデート＆重複チェック（routed）
                for (let iface in jsonObj.Devices) {
                    that.ValidateCidr(jsonObj.Devices[iface].IPAddr, 'Device ' + iface, true);
                    that.CheckDuplicateIp(jsonObj.Devices[iface].IPAddr, jsonObj, 'Device ' + iface, false);
                }
                // バリデート＆重複チェック（tag vlan）
                for (let iface in jsonObj.Vlans) {
                    if (jsonObj.Vlans[iface] !== null) {
                        for (let key in jsonObj.Vlans[iface]) {
                            that.ValidateCidr(jsonObj.Vlans[iface][key].Ip, 'Vlan ' + iface + " No." + jsonObj.Vlans[iface][key].No, false);
                            that.ValidateVlanId(jsonObj.Vlans[iface][key].Id, 'Vlan ' + iface + " No." + jsonObj.Vlans[iface][key].No);
                            that.ValidateVlanNo(jsonObj.Vlans[iface][key].No, 'Vlan ' + iface + " No." + jsonObj.Vlans[iface][key].No);
                            that.CheckDuplicateIp(jsonObj.Vlans[iface][key].Ip, jsonObj, 'Vlan ' + iface + " No." + jsonObj.Vlans[iface][key].No, true);
                            that.CheckDuplicateId(jsonObj.Vlans[iface][key].Id, jsonObj.Vlans, 'Vlan ' + iface + " No." + jsonObj.Vlans[iface][key].No);
                            that.CheckDuplicateNo(jsonObj.Vlans[iface][key].No, jsonObj.Vlans[iface], 'Vlan ' + iface + " No." + jsonObj.Vlans[iface][key].No);
                        }
                    }
                }
                if (that.GetInvalidFlag()) {
                    that.ShowErrorMessage();
                    return;
                }
                that.DoRestore(jsonObj);
                $('#restore-file-field').val($('#restore-upload').val());
            }
        })(file);
        reader.readAsText(file);
    }
    /**
    * オブジェクトの精査
    * @param {typeof} type
    * @param {string} kind
    **/
    Restore.prototype.CheckObject = function(type, kind) {
        if (type === 'undefined') {
            this.SetMessages(kind + ': There is no item.');
            this.SetInvalidFlag(true);
        }
    }
    /**
    * IPアドレスチェック
    * @param {string} cidr
    * @param {string} kind
    **/
    Restore.prototype.ValidateCidr = function(cidr, kind, isRouted) {
        if ((cidr === null || cidr === "") && isRouted === true) {
            // routedのときは null もしくは "" でもでも良しとする
            return;
        }
        if (!this.validater.Cidr(cidr)) {
            this.SetMessages(kind + ': Invalid IP Address.');
            this.SetInvalidFlag(true);
        }
    }
    /**
    * VlanIdチェック
    * @param {string} vlanId
    * @param {string} kind
    **/
    Restore.prototype.ValidateVlanId = function(vlanId, kind) {
        if (vlanId === 'undefined') {
            return;
        }
        if (!this.validater.VlanId(vlanId)) {
            this.SetMessages(kind + ': Invalid Vlan ID.');
            this.SetInvalidFlag(true);
        }
    }
    /**
    * VlanNoチェック
    * @param {string} no
    * @param {string} kind
    **/
    Restore.prototype.ValidateVlanNo = function(no, kind) {
        if (no === 'undefined') {
            return;
        }
        let regexp = /^[0-9]+$/;
        if (!no.match(regexp)) {
            this.SetMessages(kind + ': Invalid Vlan No.');
            this.SetInvalidFlag(true);
        }
    }
    /**
    * IP重複チェック
    * @param {string} ip
    * @param {object} obj
    * @param {string} kind
    * @param {bool} isAll
    **/
    Restore.prototype.CheckDuplicateIp = function(ip, obj, kind, isAll) {
        let cnt = 0;
        if (typeof ip !== 'string') { return; }
        ip = ip.split('/')[0];
        // routedチェック
        for (let key in obj.Devices) {
            if (typeof obj.Devices[key].IPAddr !== 'string' || obj.Devices[key].IPAddr === "") { return; }
            let devIp = obj.Devices[key].IPAddr.split('/')[0];
            if (devIp === ip) {
                cnt++;
            }
        }
        // deviceとの重複チェックのみ行う場合
        if (!isAll && cnt > 1) {
            this.SetMessages(kind + ': Duplicate IP (' + ip + ').');
            this.SetInvalidFlag(true);
            return;
        }
        // tag vlanチェック
        for (let key in obj.Vlans) {
            for (let k in obj.Vlans[key]) {
                if (typeof obj.Vlans[key][k].Ip !== 'string') { return; }
                let vlanIp = obj.Vlans[key][k].Ip.split('/')[0];
                if (vlanIp === ip) {
                    cnt++;
                }
            }
        }
        if (cnt > 1) {
            this.SetMessages(kind + ': Duplicate IP (' + ip + ').');
            this.SetInvalidFlag(true);
        }
    }
    /**
    * ID重複チェック
    * @param {string} id
    * @param {object} obj
    * @param {string} kind
    **/
    Restore.prototype.CheckDuplicateId = function(id, obj, kind) {
        if (obj === null || obj === undefined) {
            return;
        }
        let cnt = 0;
        for (let key in obj) {
            for (let k in obj[key]) {
                if (obj[key][k].Id === id) {
                    cnt++;
                }
            }
        }
        if (cnt > 1) {
            this.SetMessages(kind + ': Duplicate Vlan ID (' + id + ').');
            this.SetInvalidFlag(true);
        }
    }

    /**
    * 番号重複チェック
    * @param {string} no
    * @param {object} obj
    * @param {string} kind
    **/
    Restore.prototype.CheckDuplicateNo = function(no, obj, kind) {
        if (obj === null || obj === undefined) {
            return;
        }
        let cnt = 0;
        for (let key in obj) {
            if (obj[key].No === no) {
                cnt++;
            }
        }
        if (cnt > 1) {
            this.SetMessages(kind + ': Duplicate Vlan No.');
            this.SetInvalidFlag(true);
        }
    }
    /**
    * エラーメッセージ表示
    **/
    Restore.prototype.ShowErrorMessage = function() {
        let msgs = this.GetMessages();
        let len = msgs.length;
        let messageHtml = "";
        for (let i = 0; i < len; i++) {
            messageHtml += msgs[i] + "<br>";
        }
        $('#modal-restore-json-invalid-errors').html(messageHtml);
        $('#modal-restore-json-invalid').css({'display': 'block'});
        this.Init();
    }
    /**
    * オブジェクト初期化
    **/
    Restore.prototype.Init = function() {
        // 初期化
        this.errMessages = [];
        this.SetFile(null);
        this.SetInvalidFlag(false);
        $('#restore-upload').val('');
        $('#restore-file-field').val('');
        hideBigLoader();
    }
    /**
    * リストア実行
    **/
    Restore.prototype.DoRestore = function(obj) {
        let that = this;
        $.ajax({
            type: 'POST',
            url: 'http://192.0.2.1:8080/1/setting/restore',
            dataType: 'json',
            contentType: 'application/json',
            data: JSON.stringify(obj),
            cache: false
        }).then(
            function(data){ // success
                // - 実行ログ書き込み
                that.Init();
                let msg = '<p class="result_label">Routed</p>';
                if (data.Devices) {
                    for (let device in data.Devices) {
                        if (!data.Devices[device]) {
                            continue
                        }
                        let icon = (data.Devices[device].indexOf('success') !== -1) ? '<i class="fa fa-check" style="color:#00a65a"></i> ' : '<i class="fa fa-times" style="color: red;"></i> ';
                        msg += icon + device + ": " + data.Devices[device] + "<br>";
                    }
                }
                if (data.Vlans) {
                    msg += '<p class="result_label">Tag Vlan</p>';
                    for (let v in data.Vlans) {
                        if (data.Vlans[v] === null) {
                            vlan.RemoveModeVlan(v.toLocaleLowerCase());
                            continue;
                        }
                        for (let index in data.Vlans[v]) {
                            let icon = (data.Vlans[v][index].indexOf('success') !== -1) ? '<i class="fa fa-check" style="color:#00a65a"></i> ' : '<i class="fa fa-times" style="color: red;"></i> ';
                            msg += icon + v + ": " + data.Vlans[v][index] + "<br>";
                        }
                    }
                }
                $('#restore-detail-area').html(msg);
                $('#all-settings-backup').removeAttr('disabled', 'disabled');
                $('#restore-browse').removeAttr('disabled', 'disabled');
            },
            function(){ // err
                that.Init();
                $('#restore-detail-area').html('Error: Failed to connect');
                $('#all-settings-backup').removeAttr('disabled', 'disabled');
                $('#restore-browse').removeAttr('disabled', 'disabled');
            }
        );
    }
    // - API送信
    // - restore log 出力
}

// リストアボタンクリック
$(document).on('click', '#restore-browse', function() {
    $('#modal-restore-warning').css({'display': 'block'});
});
// リストアアラートのOKボタンクリック
$(document).on('click', '#modal-restore-warning-ok', function(){
    $('#restore-upload').click();
    $('#modal-restore-warning').css({'display': 'none'});
});
// ファイル選択されたとき
$('#restore-upload').change(function(event){
    restore.CheckFileFormet(event);
});
$(document).on('click', '#modal-restore-json-invalid-close', function(){
    $('#modal-restore-json-invalid').css({'display': 'none'});
})